/*
 *
 *  Copyright (C) 1997-2010, OFFIS e.V.
 *  All rights reserved.  See COPYRIGHT file for details.
 *
 *  This software and supporting documentation were developed by
 *
 *    OFFIS e.V.
 *    R&D Division Health
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *
 *  Module:  dcmjp2k
 *
 *  Author:  Marco Eichelberg
 *
 *  Purpose: codec parameter class for JasPer based JPEG 2000 codecs
 *
 *  Last Update:      $Author$
 *  Update Date:      $Date$
 *  CVS/RCS Revision: $Revision$
 *  Status:           $State$
 *
 *  CVS/RCS Log at end of file
 *
 */

#ifndef D2CPARAM_H
#define D2CPARAM_H

#include "dcmtk/config/osconfig.h"
#include "dcmtk/dcmdata/dccodec.h" /* for DcmCodecParameter */
#include "dcmtk/dcmjp2k/d2utils.h" /* for enums */

/** codec parameter for JasPer based JPEG 2000 codecs
 */
class D2CodecParameter: public DcmCodecParameter
{
public:

  /** constructor.
   *  @param opt_quality_layers        compression ratios for intermediate quality layers
   *  @param imageAreaOffsetX          x-coordinate of the top-left corner of the image area
   *  @param imageAreaOffsetY          y-coordinate of the top-left corner of the image area
   *  @param tileGridOffsetX           x-coordinate of the top-left corner of the tiling grid
   *  @param tileGridOffsetY           y-coordinate of the top-left corner of the tiling grid
   *  @param tileWidth                 nominal tile width
   *  @param tileHeight                nominal tile height
   *  @param codeBlockWidth            base 2 logarithm of the nominal code block width (2..10). Default is 6.
   *  @param codeBlockHeight           base 2 logarithm of the nominal code block height (2..10). Default is 6.
   *  @param precinctWidth             base 2 logarithm of the precinct width (0..15). Default is 15.
   *  @param precinctHeight            base 2 logarithm of the precinct height (0..15). Default is 15.
   *  @param generateSOP               coding style: Generate SOP marker segments
   *  @param generateEPH               coding style: Generate EPH marker segments
   *  @param useSegSymbols             code block style: Use segmentation symbols.
   *  @param usePredictableTermination code block style: Use predictable termination.
   *  @param progression               progression order, default is EJ2O_progression_lrcp.
   *  @param resolutionLevels          number of resolution levels, must be > 0. Default is 6.
   *  @param guardBits                 number of guard bits, must be <= 8. Default is 2.
   *  @param lazyCoding                code block style: Enable lazy coding mode (a.k.a. arithmetic coding bypass).
   *  @param terminateAllPasses        code block style: Terminate all coding passes.
   *  @param useVerticalCausalContexts code block style: Vertically stripe causal contexts.
   *  @param resetProbabilityModels    code block style: Reset the probability models after each coding pass.
   *  @param colorSpaceConversion      Multi-component (i.e. color space) transformations
   *  @param convertToMonochrome       convert color images to monochrome (lossy only)
   *  @param preferCookedEncoding      true if the "cooked" lossless encoder should be preferred over the "raw" one
   *  @param compressionBithDepth      strategy for compressed bit depth in lossy compression
   *  @param bitDepthLimit             limit for bit compressed bit depth in lossy compression, depending on strategy
   *  @param windowType                mode for VOI transformation of monochrome images
   *  @param windowParameter           parameter for VOI transform of monochrome images, used in modes 1, 2, 4, 6
   *  @param voiCenter                 VOI window center for mode 5
   *  @param voiWidth                  VOI window width for mode 5
   *  @param roiLeft                   Region of Interest left corner for for VOI transform of monochrome images, mode 7
   *  @param roiTop                    Region of Interest upper corner for for VOI transform of monochrome images, mode 7
   *  @param roiWidth                  Region of Interest width for for VOI transform of monochrome images, mode 7
   *  @param roiHeight                 Region of Interest height for for VOI transform of monochrome images, mode 7
   *  @param usePixelValues            Check smallest and largest pixel value and optimize compression, mode 0 only
   *  @param useModalityRescale        Create Rescale Slope/Intercept to scale back to original pixel range, mode 0 only
   *  @param fragmentSize              maximum fragment size (in kbytes) for compression, 0 for unlimited.
   *  @param createOffsetTable         create offset table during image compression
   *  @param uidCreation               mode for SOP Instance UID creation (used both for encoding and decoding)
   *  @param convertToSC               flag indicating whether image should be converted to Secondary Capture upon compression
   *  @param acceptWrongPaletteTags    accept incorrect palette attribute tags (0028,111x) and (0028,121x) (lossy compression only)
   *  @param acrNemaCompatibility      accept ACR-NEMA images without photometric interpretation (lossy compression only)
   *  @param planarConfiguration       flag describing how planar configuration of decompressed color images should be handled
   */
  D2CodecParameter(
    const OFList< double > & opt_quality_layers,
    Uint32 imageAreaOffsetX = 0,
    Uint32 imageAreaOffsetY = 0,
    Uint32 tileGridOffsetX = 0,
    Uint32 tileGridOffsetY = 0,
    Uint32 tileWidth = 0,
    Uint32 tileHeight = 0,
    Uint8 codeBlockWidth = 6,
    Uint8 codeBlockHeight = 6,
    Uint8 precinctWidth = 15,
    Uint8 precinctHeight = 15,
    OFBool generateSOP = OFFalse,
    OFBool generateEPH = OFFalse,
    OFBool useSegSymbols = OFFalse,
    OFBool usePredictableTermination = OFFalse,
    J2K_Progression progression = EJ2O_progression_lrcp,
    Uint8 resolutionLevels = 6,
    Uint8 guardBits = 2,
    OFBool lazyCoding = OFFalse,
    OFBool terminateAllPasses = OFFalse,
    OFBool useVerticalCausalContexts = OFFalse,
    OFBool resetProbabilityModels = OFFalse,
    OFBool colorSpaceConversion = OFTrue,
    OFBool convertToMonochrome = OFFalse,
    OFBool preferCookedEncoding = OFTrue,
    J2K_CompressionBitDepth compressionBithDepth = EJ2BD_limit,
    Uint32 bitDepthLimit = 16,
    unsigned long windowType = 0,
    unsigned long windowParameter = 0,
    double voiCenter = 0.0,
    double voiWidth = 0.0,
    unsigned long roiLeft = 0,
    unsigned long roiTop = 0,
    unsigned long roiWidth = 0,
    unsigned long roiHeight = 0,
    OFBool usePixelValues = OFTrue,
    OFBool useModalityRescale = OFFalse,
    Uint32 fragmentSize = 0,
    OFBool createOffsetTable = OFTrue,
    J2K_UIDCreation uidCreation = EJ2UC_default,
    OFBool convertToSC = OFFalse,
    OFBool acceptWrongPaletteTags = OFFalse, 
    OFBool acrNemaCompatibility = OFFalse,
    J2K_PlanarConfiguration planarConfiguration = EJ2PC_restore);

  /** constructor, for use with decoders. Initializes all encoder options to defaults.
   *  @param uidCreation               mode for SOP Instance UID creation (used both for encoding and decoding)
   *  @param planarConfiguration       flag describing how planar configuration of decompressed color images should be handled
   */
  D2CodecParameter(
    J2K_UIDCreation uidCreation = EJ2UC_default,
    J2K_PlanarConfiguration planarConfiguration = EJ2PC_restore);

  /// copy constructor
  D2CodecParameter(const D2CodecParameter& arg);

  /// destructor
  virtual ~D2CodecParameter();

  /** this methods creates a copy of type DcmCodecParameter *
   *  it must be overweritten in every subclass.
   *  @return copy of this object
   */
  virtual DcmCodecParameter *clone() const;

  /** returns the class name as string.
   *  can be used as poor man's RTTI replacement.
   */
  virtual const char *className() const;


  /** returns maximum fragment size (in kbytes) for compression, 0 for unlimited.
   *  @returnmaximum fragment size for compression
   */
  Uint32 getFragmentSize() const
  {
    return fragmentSize_;
  }

  /** returns offset table creation flag
   *  @return offset table creation flag
   */
  OFBool getCreateOffsetTable() const
  {
    return createOffsetTable_;
  }

  /** returns secondary capture conversion flag
   *  @return secondary capture conversion flag
   */
  OFBool getConvertToSC() const
  {
    return convertToSC_;
  }

  /** returns mode for SOP Instance UID creation
   *  @return mode for SOP Instance UID creation
   */
  J2K_UIDCreation getUIDCreation() const
  {
    return uidCreation_;
  }

  /** returns mode for handling planar configuration
   *  @return mode for handling planar configuration
   */
  J2K_PlanarConfiguration getPlanarConfiguration() const
  {
    return planarConfiguration_;
  }

  /** returns mode for VOI transformation of monochrome images.
   *  @return mode for VOI transformation of monochrome images
   */
  unsigned long getWindowType() const
  {
    return windowType_;
  }

  /** returns parameter for VOI transform of monochrome images, used in VOI modes 1, 2, 4, 6
   *  @return parameter for VOI transform of monochrome images, used in VOI modes 1, 2, 4, 6
   */
  unsigned long getWindowParameter() const
  {
    return windowParameter_;
  }

  /** returns VOI window center and width for VOI mode 5
   *  @param center VOI center returned in this parameter
   *  @param width VOI width returned in this parameter
   */
  void getVOIWindow(double& center, double& width) const
  {
    center = voiCenter_;
    width = voiWidth_;
  }

  /** returns ROI coordinates for VOI mode 7
   *  @param left ROI left edge returned in this parameter
   *  @param top ROI top edge returned in this parameter
   *  @param width ROI width returned in this parameter
   *  @param height ROI height returned in this parameter
   */
  void getROI(
    unsigned long& left,
    unsigned long& top,
    unsigned long& width,
    unsigned long& height) const
  {
    left = roiLeft_;
    top = roiTop_;
    width = roiWidth_;
    height = roiHeight_;
  }

  /** returns flag indicating if compression to be optimized for real pixel values
   *  @return flag indicating if compression to be optimized for real pixel values
   */
  OFBool getUsePixelValues() const
  {
    return usePixelValues_;
  }

  /** returns flag indicating if compression should compute Rescale Slope/Intercept
   *  @return flag indicating if compression should compute Rescale Slope/Intercept
   */
  OFBool getUseModalityRescale() const
  {
    return useModalityRescale_;
  }

  /** returns flag indicating if color images should be converted to monochrome upon lossy compression
   *  @return monochrome conversion flag
   */
  OFBool convertToMonochrome() const
  {
    return convertToMonochrome_;
  }

  /** returns flag indicating whether or not the multi-component transformation is
   *  endabled.
   *  @return multi-component transformation enabled flag
   */
  OFBool multiComponentTransformationEnabled() const
  {
    return colorSpaceConversion_;
  }

  /** returns flag indicating whether or not the "cooked" lossless encoder 
   *  should be preferred over the "raw" one
   *  @return raw/cooked lossless encoding flag
   */
  OFBool cookedEncodingPreferred() const
  {
    return preferCookedEncoding_;
  }

 
  /** returns flag indicating the strategy for handling bit
   *  depth upon lossy compression
   *  @return lossy compression bit depth strategy flag
   */
  J2K_CompressionBitDepth getCompressionBithDepthStrategy() const
  {
    return compressionBithDepth_;
  }

  /** returns bit depth limit for lossy compression.
   *  The interpretation of this value depends on the compression bit depth strategy.
   *  @return lossy compression bit depth limit
   */
  Uint32 getCompressionBithDepthLimit() const
  {
    return bitDepthLimit_;
  }

  /** create a string containing the codec parameters in the format
   *  expected by the JasPer library.
   *  @param parms string returned in this parameter
   */
  void getCodecParamString(OFString& parms) const;


  /** returns flag indicating whether to accept incorrect palette 
   *  attribute tags (0028,111x) and (0028,121x) (lossy compression only)
   *  @return acceptWrongPaletteTags flag
   */
  OFBool getAcceptWrongPaletteTags() const
  {
    return acceptWrongPaletteTags_;
  }

  /** returns flag indicating whether to accept ACR-NEMA images without 
   *  photometric interpretation (lossy compression only)
   *  @return acrNemaCompatibility flag
   */
  OFBool getAcrNemaCompatibility() const
  {
    return acrNemaCompatibility_;
  }

private:

  /// private undefined copy assignment operator
  D2CodecParameter& operator=(const D2CodecParameter&);

  // ****************************************************
  // **** Parameters describing the encoding process ****

  /// x-coordinate of the top-left corner of the image area
  Uint32 imageAreaOffsetX_;

  /// y-coordinate of the top-left corner of the image area
  Uint32 imageAreaOffsetY_;

  /// x-coordinate of the top-left corner of the tiling grid
  Uint32 tileGridOffsetX_;

  /// y-coordinate of the top-left corner of the tiling grid
  Uint32 tileGridOffsetY_;

  /// nominal tile width
  Uint32 tileWidth_;

  /// nominal tile height
  Uint32 tileHeight_;

  /// base 2 logarithm of the nominal code block width (2..10). Default is 6.
  Uint8 codeBlockWidth_;

  /// base 2 logarithm of the nominal code block height (2..10). Default is 6.
  Uint8 codeBlockHeight_;

  /// base 2 logarithm of the precinct width (0..15). Default is 15.
  Uint8 precinctWidth_;

  /// base 2 logarithm of the precinct height (0..15). Default is 15.
  Uint8 precinctHeight_;

  /// coding style: Generate SOP marker segments
  OFBool generateSOP_;

  /// coding style: Generate EPH marker segments
  OFBool generateEPH_;

  /// code block style: Use segmentation symbols.
  OFBool useSegSymbols_;

  /// code block style: Use predictable termination.
  OFBool usePredictableTermination_;

  /// progression order, default is EJ2O_progression_lrcp.
  J2K_Progression progression_;

  /// number of resolution levels, must be > 0. Default is 6.
  Uint8 resolutionLevels_;

  /// number of guard bits, must be <= 8. Default is 2.
  Uint8 guardBits_;

  /// code block style: Enable lazy coding mode (a.k.a. arithmetic coding bypass).
  OFBool lazyCoding_;

  /// code block style: Terminate all coding passes.
  OFBool terminateAllPasses_;

  /// code block style: Vertically stripe causal contexts.
  OFBool useVerticalCausalContexts_;

  /// code block style: Reset the probability models after each coding pass.
  OFBool resetProbabilityModels_;

  /// compression ratios for intermediate quality layers
  OFList< double > opt_quality_layers_;

  /// Multi-component (i.e. color space) transformations
  OFBool colorSpaceConversion_;

  /// Convert color images to monochrome (lossy only)
  OFBool convertToMonochrome_;
  
  /// Flag indicating if the "cooked" lossless encoder should be preferred over the "raw" one
  OFBool preferCookedEncoding_;

  /// strategy for compressed bit depth in lossy compression
  J2K_CompressionBitDepth compressionBithDepth_;

  /// limit for bit compressed bit depth in lossy compression, depending on strategy
  Uint32 bitDepthLimit_;

  /// mode for VOI transformation of monochrome images
  unsigned long windowType_;

  /// parameter for VOI transform of monochrome images, used in modes 1, 2, 4, 6
  unsigned long windowParameter_;

  /// VOI window center for mode 5
  double voiCenter_;

  /// VOI window width for mode 5
  double voiWidth_;

  /// Region of Interest left corner for for VOI transform of monochrome images, mode 7
  unsigned long roiLeft_;

  /// Region of Interest upper corner for for VOI transform of monochrome images, mode 7
  unsigned long roiTop_;

  /// Region of Interest width for for VOI transform of monochrome images, mode 7
  unsigned long roiWidth_;

  /// Region of Interest height for for VOI transform of monochrome images, mode 7
  unsigned long roiHeight_;

  /// Check smallest and largest pixel value and optimize compression, mode 0 only
  OFBool usePixelValues_;

  /// Create Rescale Slope/Intercept to scale back to original pixel range, mode 0 only
  OFBool useModalityRescale_;

  /// maximum fragment size (in kbytes) for compression, 0 for unlimited.
  Uint32 fragmentSize_;

  /// create offset table during image compression
  OFBool createOffsetTable_;

  /// mode for SOP Instance UID creation (used both for encoding and decoding)
  J2K_UIDCreation uidCreation_;

  /// flag indicating whether image should be converted to Secondary Capture upon compression
  OFBool convertToSC_;

  /// flag indicating whether to accept incorrect palette attribute tags (0028,111x) and (0028,121x) (lossy compression only)
  OFBool acceptWrongPaletteTags_;

  /// flag indicating whether to accept ACR-NEMA images without photometric interpretation (lossy compression only)
  OFBool acrNemaCompatibility_;

  // ****************************************************
  // **** Parameters describing the decoding process ****

  /// flag describing how planar configuration of decompressed color images should be handled
  J2K_PlanarConfiguration planarConfiguration_;

};


#endif

/*
 * CVS/RCS Log
 * $Log$
 * Revision 1.5  2010-10-14 09:28:07  joergr
 * Updated copyright header. Added reference to COPYRIGHT file.
 *
 * Revision 1.4  2009-10-07 14:23:37  uli
 * Switched to logging mechanism provided by the "new" oflog module.
 *
 * Revision 1.3  2005-12-08 16:59:01  meichel
 * Changed include path schema for all DCMTK header files
 *
 * Revision 1.2  2005/11/30 16:27:30  meichel
 * Added commandline options --accept-acr-nema and --accept-palettes
 *   (same as in dcm2pnm) to dcmcjp2k
 *
 * Revision 1.1  2004/10/20 15:48:54  meichel
 * Initial release of module dcmjp2k, a JPEG 2000 codec for DCMTK.
 *
 *
 */
