/*
 *
 *  Copyright (C) 1997-2010, OFFIS e.V.
 *  All rights reserved.  See COPYRIGHT file for details.
 *
 *  This software and supporting documentation were developed by
 *
 *    OFFIS e.V.
 *    R&D Division Health
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *
 *  Module:  dcmjp2k
 *
 *  Author:  Marco Eichelberg
 *
 *  Purpose: singleton class that registers encoders for all supported JPEG 2000 processes.
 *
 *  Last Update:      $Author$
 *  Update Date:      $Date$
 *  CVS/RCS Revision: $Revision$
 *  Status:           $State$
 *
 *  CVS/RCS Log at end of file
 *
 */

#ifndef D2ENCODE_H
#define D2ENCODE_H

#include "dcmtk/config/osconfig.h"
#include "dcmtk/ofstd/oftypes.h"  /* for OFBool */
#include "dcmtk/ofstd/oflist.h"
#include "dcmtk/dcmjp2k/d2utils.h"
#include "dcmtk/dcmdata/dctypes.h"  /* for Uint32 */

class D2CodecParameter;
class D2JPEG2000LossyEncoder;
class D2JPEG2000LosslessEncoder;

/** singleton class that registers encoders for all supported JPEG 2000 processes.
 */
class D2EncoderRegistration 
{
public: 

  /** registers encoders for all supported JPEG 2000 processes.
   *  If already registered, call is ignored unless cleanup() has
   *  been performed before.
   *  @param opt_quality_layers        compression ratios for intermediate quality layers
   *  @param imageAreaOffsetX          x-coordinate of the top-left corner of the image area
   *  @param imageAreaOffsetY          y-coordinate of the top-left corner of the image area
   *  @param tileGridOffsetX           x-coordinate of the top-left corner of the tiling grid
   *  @param tileGridOffsetY           y-coordinate of the top-left corner of the tiling grid
   *  @param tileWidth                 nominal tile width
   *  @param tileHeight                nominal tile height
   *  @param codeBlockWidth            base 2 logarithm of the nominal code block width (2..10). Default is 6.
   *  @param codeBlockHeight           base 2 logarithm of the nominal code block height (2..10). Default is 6.
   *  @param precinctWidth             base 2 logarithm of the precinct width (0..15). Default is 15.
   *  @param precinctHeight            base 2 logarithm of the precinct height (0..15). Default is 15.
   *  @param generateSOP               coding style: Generate SOP marker segments
   *  @param generateEPH               coding style: Generate EPH marker segments
   *  @param useSegSymbols             code block style: Use segmentation symbols.
   *  @param usePredictableTermination code block style: Use predictable termination.
   *  @param progression               progression order, default is EJ2O_progression_lrcp.
   *  @param resolutionLevels          number of resolution levels, must be > 0. Default is 6.
   *  @param guardBits                 number of guard bits, must be <= 8. Default is 2.
   *  @param lazyCoding                code block style: Enable lazy coding mode (a.k.a. arithmetic coding bypass).
   *  @param terminateAllPasses        code block style: Terminate all coding passes.
   *  @param useVerticalCausalContexts code block style: Vertically stripe causal contexts.
   *  @param resetProbabilityModels    code block style: Reset the probability models after each coding pass.
   *  @param colorSpaceConversion      Multi-component (i.e. color space) transformations
   *  @param convertToMonochrome       convert color images to monochrome (lossy only)
   *  @param preferCookedEncoding      true if the "cooked" lossless encoder should be preferred over the "raw" one
   *  @param compressionBithDepth      strategy for compressed bit depth in lossy compression
   *  @param bitDepthLimit             limit for bit compressed bit depth in lossy compression, depending on strategy
   *  @param windowType                mode for VOI transformation of monochrome images
   *  @param windowParameter           parameter for VOI transform of monochrome images, used in modes 1, 2, 4, 6
   *  @param voiCenter                 VOI window center for mode 5
   *  @param voiWidth                  VOI window width for mode 5
   *  @param roiLeft                   Region of Interest left corner for for VOI transform of monochrome images, mode 7
   *  @param roiTop                    Region of Interest upper corner for for VOI transform of monochrome images, mode 7
   *  @param roiWidth                  Region of Interest width for for VOI transform of monochrome images, mode 7
   *  @param roiHeight                 Region of Interest height for for VOI transform of monochrome images, mode 7
   *  @param usePixelValues            Check smallest and largest pixel value and optimize compression, mode 0 only
   *  @param useModalityRescale        Create Rescale Slope/Intercept to scale back to original pixel range, mode 0 only
   *  @param fragmentSize              maximum fragment size (in kbytes) for compression, 0 for unlimited.
   *  @param createOffsetTable         create offset table during image compression
   *  @param uidCreation               mode for SOP Instance UID creation
   *  @param convertToSC               flag indicating whether image should be converted to Secondary Capture upon compression
   *  @param acceptWrongPaletteTags    accept incorrect palette attribute tags (0028,111x) and (0028,121x) (lossy compression only)
   *  @param acrNemaCompatibility      accept ACR-NEMA images without photometric interpretation (lossy compression only)
   */
  static void registerCodecs(
    const OFList< double > & opt_quality_layers = OFList< double > (),
    Uint32 imageAreaOffsetX = 0,
    Uint32 imageAreaOffsetY = 0,
    Uint32 tileGridOffsetX = 0,
    Uint32 tileGridOffsetY = 0,
    Uint32 tileWidth = 0,
    Uint32 tileHeight = 0,
    Uint8 codeBlockWidth = 6,
    Uint8 codeBlockHeight = 6,
    Uint8 precinctWidth = 15,
    Uint8 precinctHeight = 15,
    OFBool generateSOP = OFFalse,
    OFBool generateEPH = OFFalse,
    OFBool useSegSymbols = OFFalse,
    OFBool usePredictableTermination = OFFalse,
    J2K_Progression progression = EJ2O_progression_lrcp,
    Uint8 resolutionLevels = 6,
    Uint8 guardBits = 2,
    OFBool lazyCoding = OFFalse,
    OFBool terminateAllPasses = OFFalse,
    OFBool useVerticalCausalContexts = OFFalse,
    OFBool resetProbabilityModels = OFFalse,
    OFBool colorSpaceConversion = OFTrue,
    OFBool convertToMonochrome = OFFalse,
    OFBool preferCookedEncoding = OFTrue,
    J2K_CompressionBitDepth compressionBithDepth = EJ2BD_limit,
    Uint32 bitDepthLimit = 16,
    unsigned long windowType = 0,
    unsigned long windowParameter = 0,
    double voiCenter = 0.0,
    double voiWidth = 0.0,
    unsigned long roiLeft = 0,
    unsigned long roiTop = 0,
    unsigned long roiWidth = 0,
    unsigned long roiHeight = 0,
    OFBool usePixelValues = OFTrue,
    OFBool useModalityRescale = OFFalse,
    Uint32 fragmentSize = 0,
    OFBool createOffsetTable = OFTrue,
    J2K_UIDCreation uidCreation = EJ2UC_default,
    OFBool convertToSC = OFFalse,
    OFBool acceptWrongPaletteTags = OFFalse, 
    OFBool acrNemaCompatibility = OFFalse);

  /** deregisters encoders.
   *  Attention: Must not be called while other threads might still use
   *  the registered codecs, e.g. because they are currently encoding
   *  DICOM data sets through dcmdata.
   */  
  static void cleanup();

private:

  /// flag indicating whether the encoders are already registered.
  static OFBool registered_;

  /// pointer to codec parameter shared by all encoders
  static D2CodecParameter *cp_;
  
  /// pointer to encoder for lossy JPEG 2000
  static D2JPEG2000LossyEncoder  *lossyencoder_;

  /// pointer to encoder for lossless JPEG 2000
  static D2JPEG2000LosslessEncoder *losslessencoder_;
  
};

#endif

/*
 * CVS/RCS Log
 * $Log$
 * Revision 1.5  2010-10-14 09:28:07  joergr
 * Updated copyright header. Added reference to COPYRIGHT file.
 *
 * Revision 1.4  2009-10-07 14:23:37  uli
 * Switched to logging mechanism provided by the "new" oflog module.
 *
 * Revision 1.3  2005-12-08 16:59:03  meichel
 * Changed include path schema for all DCMTK header files
 *
 * Revision 1.2  2005/11/30 16:27:30  meichel
 * Added commandline options --accept-acr-nema and --accept-palettes
 *   (same as in dcm2pnm) to dcmcjp2k
 *
 * Revision 1.1  2004/10/20 15:48:54  meichel
 * Initial release of module dcmjp2k, a JPEG 2000 codec for DCMTK.
 *
 *
 */
