/*
 *
 *  Copyright (C) 1997-2010, OFFIS e.V.
 *  All rights reserved.  See COPYRIGHT file for details.
 *
 *  This software and supporting documentation were developed by
 *
 *    OFFIS e.V.
 *    R&D Division Health
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *
 *  Module:  dcmjp2k
 *
 *  Author:  Marco Eichelberg
 *
 *  Purpose: representation parameters for JPEG 2000
 *
 *  Last Update:      $Author$
 *  Update Date:      $Date$
 *  CVS/RCS Revision: $Revision$
 *  Status:           $State$
 *
 *  CVS/RCS Log at end of file
 *
 */

#ifndef D2RPARAM_H
#define D2RPARAM_H

#include "dcmtk/config/osconfig.h"
#include "dcmtk/dcmdata/dcpixel.h" /* for class DcmRepresentationParameter */

/** representation parameter for JPEG 2000
 */
class D2RepresentationParameter: public DcmRepresentationParameter
{
public:

  /** constructor
   *  @param compressionRatio desired compression ratio, must be <= 1.0
   *  @param compressedSize desired compressed frame size in bytes. Overrides compressionRatio if nonzero.
   *  @param losslessProcess true if lossless process is requested
   */
  D2RepresentationParameter(
    double compressionRatio,
    Uint32 compressedSize,
    OFBool losslessProcess,
    const OFString &imageComment = "");

  /// copy constructor
  D2RepresentationParameter(const D2RepresentationParameter& arg);

  /// destructor
  virtual ~D2RepresentationParameter();
  
  /** this methods creates a copy of type DcmRepresentationParameter *
   *  it must be overweritten in every subclass.
   *  @return copy of this object
   */
  virtual DcmRepresentationParameter *clone() const;

  /** returns the class name as string.
   *  can be used in operator== as poor man's RTTI replacement.
   */
  virtual const char *className() const;

  /** compares an object to another DcmRepresentationParameter.
   *  Implementation must make sure that classes are comparable.
   *  @param arg representation parameter to compare with
   *  @return true if equal, false otherwise.
   */
  virtual OFBool operator==(const DcmRepresentationParameter &arg) const;

  /** returns the desired compression ratio
   *  @return desired compression ratio
   */
  double getCompressionRatio() const
  {
    return compressionRatio_;
  }

  /** returns the desired compressed frame size in bytes
   *  @return desired compressed frame size in bytes
   */
  Uint32 getCompressedSize() const
  {
    return compressedSize_;
  }

  /** returns true if lossless compression is desired
   *  @return true if lossless compression is desired
   */
  OFBool useLosslessProcess() const
  {
    return losslessProcess_;
  }

  /** returns the descired image comment
   *  @return the descired image comment
   */
  const OFString &imageComment() const
  {
    return imageComment_;
  }

  /** add representation parameters to parameter string in the format
   *  expected by the JasPer library. This method does not erase the string
   *  but expects the string to either be empty or already contain parameters
   *  for example from the codec parameters object.
   *  @param isLosslessTS true if encoding takes place for JPEG 2000 lossless only transfer syntax
   *  @param parms string returned in this parameter
   */
  void completeCodecParamString(OFBool isLosslessTS, OFString& parms) const;

private:

  /** desired compression ratio, must be <= 1.0.
   *  Default is 0.0 (unlimited).
   *  Ignored during lossless compression.
   */
  double compressionRatio_;

  /** desired compressed frame size in bytes.
   *  Default is 0 (unlimited). A nonzero value overrides
   *  any compression factor specified in compressionFactor.
   *  Ignored during lossless compression.
   */
  Uint32 compressedSize_;

  /// true if lossless process should be used even in lossy transfer syntax
  OFBool losslessProcess_;

  /// Comment to write into the jp2k header.
  OFString imageComment_;

};


#endif

/*
 * CVS/RCS Log
 * $Log$
 * Revision 1.4  2011-01-17 08:20:32  uli
 * Added new option --image-comment "comment" to dcmcjp2k.
 *
 * Revision 1.3  2010-10-14 09:28:07  joergr
 * Updated copyright header. Added reference to COPYRIGHT file.
 *
 * Revision 1.2  2005/12/08 16:59:06  meichel
 * Changed include path schema for all DCMTK header files
 *
 * Revision 1.1  2004/10/20 15:48:54  meichel
 * Initial release of module dcmjp2k, a JPEG 2000 codec for DCMTK.
 *
 *
 */
