/*
 *
 *  Copyright (C) 1997-2010, OFFIS e.V.
 *  All rights reserved.  See COPYRIGHT file for details.
 *
 *  This software and supporting documentation were developed by
 *
 *    OFFIS e.V.
 *    R&D Division Health
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *
 *  Module:  dcmjp2k
 *
 *  Author:  Marco Eichelberg
 *
 *  Purpose: enumerations, error constants and helper functions for dcmjp2k
 *
 *  Last Update:      $Author$
 *  Update Date:      $Date$
 *  CVS/RCS Revision: $Revision$
 *  Status:           $State$
 *
 *  CVS/RCS Log at end of file
 *
 */

#ifndef D2UTILS_H
#define D2UTILS_H

#include "dcmtk/config/osconfig.h"
#include "dcmtk/ofstd/ofcond.h"   /* for class OFCondition */
#include "dcmtk/oflog/oflog.h"


// global definitions for logging mechanism provided by the oflog module

OFLogger DCM_dcmjp2kGetLogger();

#define DCMJP2K_TRACE(msg) OFLOG_TRACE(DCM_dcmjp2kGetLogger(), msg)
#define DCMJP2K_DEBUG(msg) OFLOG_DEBUG(DCM_dcmjp2kGetLogger(), msg)
#define DCMJP2K_INFO(msg)  OFLOG_INFO(DCM_dcmjp2kGetLogger(), msg)
#define DCMJP2K_WARN(msg)  OFLOG_WARN(DCM_dcmjp2kGetLogger(), msg)
#define DCMJP2K_ERROR(msg) OFLOG_ERROR(DCM_dcmjp2kGetLogger(), msg)
#define DCMJP2K_FATAL(msg) OFLOG_FATAL(DCM_dcmjp2kGetLogger(), msg)


/** describes the JPEG 2000 progression orders
 */
enum J2K_Progression
{
  /// layer-resolution-component-position (LRCP) progressive (i.e., rate scalable)
  EJ2O_progression_lrcp,

  /// resolution-layer-component-position (RLCP) progressive (i.e., resolution scalable)
  EJ2O_progression_rlcp,

  /** resolution-position-component-layer (RPCL) progressive.
   *  Not valid for all possible image geometries.
   */
  EJ2O_progression_rpcl,

  /** position-component-resolution-layer (PCRL) progressive
   *  Not valid for all possible image geometries.
   */
  EJ2O_progression_pcrl,

  /// component-position-resolution-layer (CPRL) progressive
  EJ2O_progression_cprl
};

/** describes the condition under which a compressed or decompressed image 
 *  receives a new SOP instance UID.
 */
enum J2K_UIDCreation
{
  /** Upon compression, assign new SOP instance UID if compression is lossy.
   *  Upon decompression never assign new SOP instance UID.
   */
  EJ2UC_default,

  /// always assign new SOP instance UID on compression and decompression
  EJ2UC_always,

  /// never assign new SOP instance UID
  EJ2UC_never
};

/** describes how the decoder should handle planar configuration of
 *  decompressed color images.
 */
enum J2K_PlanarConfiguration
{
  /// restore planar configuration as indicated in data set
  EJ2PC_restore,

  /** automatically determine whether color-by-plane is required from
   *  the SOP Class UID and decompressed photometric interpretation
   */
  EJ2PC_auto,
   
  /// always create color-by-pixel planar configuration
  EJ2PC_colorByPixel,
  
  /// always create color-by-plane planar configuration
  EJ2PC_colorByPlane
};

/** describes how the encoder handles the image bit depth
 *  upon lossy compression.
 */
enum J2K_CompressionBitDepth
{
  /// keep original bit depth
  EJ2BD_original,

  /** limit bit depth to a certain value, i.e. scale down
   *  if the original image bit depth is larger
   */
  EJ2BD_limit,

  /** force bit depth to a certain value, i.e. scale up
   *  or scale down the original image to match the given
   *  bit depth.
   */
  EJ2BD_force
};


// CONDITION CONSTANTS

/// error condition constant: JPEG 2000 codec (JasPer) error
extern const OFCondition EC_J2KCodecError;
/// error condition constant: Mismatch between DICOM and JPEG 2000 image width/height
extern const OFCondition EC_J2KBitstreamGeometryMismatch;
/// error condition constant: Mismatch between DICOM and JPEG 2000 color model
extern const OFCondition EC_J2KBitstreamColorModelMismatch;
/// error condition constant: Mismatch between DICOM and JPEG 2000 image bit depth
extern const OFCondition EC_J2KBitstreamPrecisionMismatch;
/// error condition constant: JPEG 2000 matrix operation error
extern const OFCondition EC_J2KMatrixOperationError;
/// error condition constant: Unsupported JPEG 2000 color model
extern const OFCondition EC_J2KUnsupportedColorModel;
/// error condition constant: Unsupported JPEG 2000 encoding
extern const OFCondition EC_J2KUnsupportedEncoding;
/// error condition constant: Unsupported bit depth in JPEG 2000 transfer syntax
extern const OFCondition EC_J2KUnsupportedBitDepth;
/// error condition constant: Bit depth too large for lossless JPEG 2000 compression"
extern const OFCondition EC_J2KBitDepthTooLarge;

#endif

/*
 * CVS/RCS Log
 * $Log$
 * Revision 1.5  2010-10-14 09:28:07  joergr
 * Updated copyright header. Added reference to COPYRIGHT file.
 *
 * Revision 1.4  2010-08-20 16:01:16  meichel
 * Added support for oflog logging
 *
 * Revision 1.3  2005/12/08 16:59:07  meichel
 * Changed include path schema for all DCMTK header files
 *
 * Revision 1.2  2004/11/11 12:31:47  meichel
 * Added doxygen documentation for error condition constants
 *
 * Revision 1.1  2004/10/20 15:48:54  meichel
 * Initial release of module dcmjp2k, a JPEG 2000 codec for DCMTK.
 *
 *
 */
